#!/usr/bin/ruby
require "rexml/document"
require "set"
include REXML

doc = nil
File.open "Intrinsics.xml.old" do |file|
	doc = Document.new file
end


diffTechs = Set.new

doc.each_recursive do |node|
   	node.name=node.name.downcase
	node.attributes.each_attribute{ |el| el.name=el.name.downcase }
end


class REXML::Element
	def remove_text
		each() do |child|
			child.remove if child.class == REXML::Text
		end
		self
	end
end

codeKeywords = Regexp.union(/:=/, /\.\.\./, / = /, /^if/, /^else/, /^switch/ )

# Add the different techs to the list and reorder the list to how we like it
doc.root.elements.each("/*/intrinsic") do |el|
	diffTechs.add el.attributes["TECH"] 
	el.remove_text
	elem = el.elements["base"]
	el.add_attribute( "base", elem.text ) if elem
	elem.remove if elem

	elem = el.elements["rettype"]
	el.add_attribute( "rettype", elem.text ) if elem
	elem.remove if elem
	elem = el.elements["suffix"]
	el.add_attribute( "suffix", elem.text ) if elem
	elem.remove if elem

	#Lets fix all the mnemonics
	el.elements.each("mnemonic") do |mn|
		text = mn.get_text
		if( text && text === "Sequence" )
			el.add_attribute( "sequence", "true" )
			mn.remove
		else
			mn.add_attribute( "base", text.to_s )
			text.remove if text
		end
	end

	#Now the hard part... fixing all the descriptions
	last = nil
	last_code = nil;
	el.elements.each("description") do |desc|
		text = desc.get_text.to_s
		#Check if it's just a mnemonic description
		if desc.text =~ /^Mnemonic:/
			mnemonic = text.split[1]
			newEl = Element.new( "parameters" )
			newEl.add_text text.sub( "Mnemonic: " + mnemonic + " ", "" )
			el.elements["mnemonic[@base='"+mnemonic+"']"].add_element newEl
			desc.remove
		else
			code = text =~ codeKeywords
			if code
				last = nil
				if last_code
					last_code.add_text "\n"+text
					desc.remove
				else
					desc.add_attribute( "code", "true" ) if (text =~ codeKeywords)
					last_code = desc
				end
			else
				last_code = nil
				if last
					last.add_text " "+text
					desc.remove
				else
					last = desc
				end
			end
		end
	end
end






diffTechs.each do |tech|
	root = doc.root.clone
	doc.root.elements.each("/*/intrinsic[@TECH='"+tech+"']") do |el|
		root.add_element el
	end
	f = File.new tech + ".xml", "w"
	root.write( f, 0 )
	f.close
end


